@@ -9,28 +9,73 @@ from os.path import join, isfile, islink
 
 import numpy as np
 
-from ase.data import chemical_symbols
 from ase.data import atomic_numbers
-from ase.units import Bohr, Hartree
+from ase.units import Bohr, Hartree, fs
+from ase.data import chemical_symbols
+from ase.io.abinit import read_abinit
+from ase.calculators.calculator import FileIOCalculator, Parameters, kpts2mp, \
+    ReadError
 
 
-class Abinit:
+keys_with_units = {
+    'toldfe': 'eV',
+    'tsmear': 'eV',
+    'paoenergyshift': 'eV',
+    'zmunitslength': 'Bohr',
+    'zmunitsangle': 'rad',
+    'zmforcetollength': 'eV/Ang',
+    'zmforcetolangle': 'eV/rad',
+    'zmmaxdispllength': 'Ang',
+    'zmmaxdisplangle': 'rad',
+    'ecut': 'eV',
+    'dmenergytolerance': 'eV',
+    'electronictemperature': 'eV',
+    'oneta': 'eV',
+    'onetaalpha': 'eV',
+    'onetabeta': 'eV',
+    'onrclwf': 'Ang',
+    'onchemicalpotentialrc': 'Ang',
+    'onchemicalpotentialtemperature': 'eV',
+    'mdmaxcgdispl': 'Ang',
+    'mdmaxforcetol': 'eV/Ang',
+    'mdmaxstresstol': 'eV/Ang**3',
+    'mdlengthtimestep': 'fs',
+    'mdinitialtemperature': 'eV',
+    'mdtargettemperature': 'eV',
+    'mdtargetpressure': 'eV/Ang**3',
+    'mdnosemass': 'eV*fs**2',
+    'mdparrinellorahmanmass': 'eV*fs**2',
+    'mdtaurelax': 'fs',
+    'mdbulkmodulus': 'eV/Ang**3',
+    'mdfcdispl': 'Ang',
+    'warningminimumatomicdistance': 'Ang',
+    'rcspatial': 'Ang',
+    'kgridcutoff': 'Ang',
+    'latticeconstant': 'Ang'}
+
+
+class Abinit(FileIOCalculator):
     """Class for doing ABINIT calculations.
 
     The default parameters are very close to those that the ABINIT
     Fortran code would use.  These are the exceptions::
 
-      calc = Abinit(label='abinit', xc='LDA', pulay=5, mix=0.1)
-
-    Use the set_inp method to set extra INPUT parameters::
-
-      calc.set_inp('nstep', 30)
-
+      calc = Abinit(label='abinit', xc='LDA', ecut=400, toldfe=1e-5)
     """
-    def __init__(self, label='abinit', xc='LDA', kpts=None, nbands=None,
-                 nstep=None, width=0.04*Hartree, ecut=None, charge=0,
-                 pulay=5, mix=0.1, pps='fhi', toldfe=1.0e-6
-                 ):
+
+    implemented_properties = ['energy', 'forces', 'stress', 'magmom']
+    command = 'abinis < PREFIX.files > PREFIX.log'
+
+    default_parameters = dict(
+        xc='LDA',
+        smearing=None,
+        kpts=None,
+        charge=0.0,
+        raw=None,
+        pps='fhi')
+
+    def __init__(self, restart=None, ignore_bad_restart_file=False,
+                 label='abinit', atoms=None, scratch=None, **kwargs):
         """Construct ABINIT-calculator object.
 
         Parameters
@@ -57,65 +102,282 @@ class Abinit:
         charge: float
             Total charge of the system.
             Default is 0.
-        pulay: int
+        npulayit: int
             Number of old densities to use for Pulay mixing.
-        mix: float
+        diemix: float
             Mixing parameter between zero and one for density mixing.
+        diemac: float
+            Model DIElectric MACroscopic constant.
+            The value of diemac should usually be bigger than 1.0d0,
+            on physical grounds. If you let diemac to its default value,
+            you might even never obtain the self-consistent convergence!
 
         Examples
         ========
         Use default values:
 
-        >>> h = Atoms('H', calculator=Abinit())
+        >>> h = Atoms('H', calculator=Abinit(ecut=200, toldfe=0.001))
         >>> h.center(vacuum=3.0)
         >>> e = h.get_potential_energy()
 
         """
 
-        self.nband = nbands # called nband in abinit
-        self.nstep = nstep
+        self.scratch = scratch
+        
+        self.species = None
+        self.ppp_list = None
 
-        if ecut is None:
-            raise ValueError('Planewave cutoff energy in eV (ecut) not set')
+        FileIOCalculator.__init__(self, restart, ignore_bad_restart_file,
+                                  label, atoms, **kwargs)
 
-        self.label = label#################### != out
-        self.xc = xc
-        self.kpts = kpts
-        self.width = width
-        self.ecut = ecut
-        self.charge = charge
-        self.pulay = pulay
-        self.mix = mix
-        self.pps = pps
-        self.toldfe = toldfe
-        if not pps in ['fhi', 'hgh', 'hgh.sc', 'hgh.k', 'tm', 'paw']:
-            raise ValueError('Unexpected PP identifier %s' % pps)
+    def check_state(self, atoms):
+        system_changes = FileIOCalculator.check_state(self, atoms)
+        # Ignore boundary conditions:
+        if 'pbc' in system_changes:
+            system_changes.remove('pbc')
+        return system_changes
 
-        self.converged = False
-        self.inp = {}
-        self.n_entries_int = 20 # integer entries per line
-        self.n_entries_float = 8 # float entries per line
+    def set(self, **kwargs):
+        changed_parameters = FileIOCalculator.set(self, **kwargs)
+        if changed_parameters:
+            self.reset()
 
-    def update(self, atoms):
-        if (not self.converged or
-            len(self.numbers) != len(atoms) or
-            (self.numbers != atoms.get_atomic_numbers()).any()):
+    def write_input(self, atoms, properties=None, system_changes=None):
+        """Write input parameters to files-file."""
+
+        FileIOCalculator.write_input(self, atoms, properties, system_changes)
+
+        if 'numbers' in system_changes or 'magmoms' in system_changes:
             self.initialize(atoms)
-            self.calculate(atoms)
-        elif ((self.positions != atoms.get_positions()).any() or
-              (self.pbc != atoms.get_pbc()).any() or
-              (self.cell != atoms.get_cell()).any()):
-            self.calculate(atoms)
+
+        fh = open(self.label + '.files', 'w')
+
+        fh.write('%s\n' % (self.prefix + '.in')) # input
+        fh.write('%s\n' % (self.prefix + '.txt')) # output
+        fh.write('%s\n' % (self.prefix + 'i')) # input
+        fh.write('%s\n' % (self.prefix + 'o')) # output
+        
+        # XXX:
+        # scratch files
+        #scratch = self.scratch
+        #if scratch is None:
+        #    scratch = dir
+        #if not os.path.exists(scratch):
+        #    os.makedirs(scratch)
+        #fh.write('%s\n' % (os.path.join(scratch, prefix + '.abinit')))
+        fh.write('%s\n' % (self.prefix + '.abinit'))
+        # Provide the psp files
+        for ppp in self.ppp_list:
+            fh.write('%s\n' % (ppp)) # psp file path
+
+        fh.close()
+
+        # Abinit will write to label.txtA if label.txt already exists,
+        # so we remove it if it's there:
+        filename = self.label + '.txt'
+        if os.path.isfile(filename):
+            os.remove(filename)
+
+        param = self.parameters
+        param.write(self.label + '.ase')
+
+        fh = open(self.label + '.in', 'w')
+        inp = {}
+        inp.update(param)
+        for key in ['xc', 'smearing', 'kpts', 'pps', 'raw']:
+            del inp[key]
+
+        smearing = param.get('smearing')
+        if 'tsmear' in param or 'occopt' in param:
+            assert smearing is None
+
+        if smearing is not None:
+            inp['occopt'] = {'fermi-dirac': 3,
+                             'gaussian': 7}[smearing[0].lower()]
+            inp['tsmear'] = smearing[1]
+
+        inp['natom'] = len(atoms)
+
+        if 'nbands' in param:
+            inp['nband'] = param.nbands
+            del inp['nbands']
+
+        if 'ixc' not in param:
+            inp['ixc'] = {'LDA': 7,
+                          'PBE': 11,
+                          'revPBE': 14,
+                          'RPBE': 15,
+                          'WC': 23}[param.xc]
+
+        magmoms = atoms.get_initial_magnetic_moments()
+        if magmoms.any():
+            inp['nsppol'] = 2
+            fh.write('spinat\n')
+            for n, M in enumerate(magmoms):
+                fh.write('%.14f %.14f %.14f\n' % (0, 0, M))
+        else:
+            inp['nsppol'] = 1
+
+        for key in sorted(inp.keys()):
+            value = inp[key]
+            unit = keys_with_units.get(key)
+            if unit is None:
+                fh.write('%s %s\n' % (key, value))
+            else:
+                if 'fs**2' in unit:
+                    value /= fs**2
+                elif 'fs' in unit:
+                    value /= fs
+                fh.write('%s %e %s\n' % (key, value, unit))
+
+        if param.raw is not None:
+            for line in param.raw:
+                if isinstance(line, tuple):
+                    fh.write(' '.join(['%s' % x for x in line]) + '\n')
+                else:
+                    fh.write('%s\n' % line)
+
+        fh.write('#Definition of the unit cell\n')
+        fh.write('acell\n')
+        fh.write('%.14f %.14f %.14f Angstrom\n' %  (1.0, 1.0, 1.0))
+        fh.write('rprim\n')
+        for v in atoms.cell:
+            fh.write('%.14f %.14f %.14f\n' %  tuple(v))
+
+        fh.write('chkprim 0 # Allow non-primitive cells\n')
+
+        fh.write('#Definition of the atom types\n')
+        fh.write('ntypat %d\n' % (len(self.species)))
+        fh.write('znucl')
+        for n, Z in enumerate(self.species):
+            fh.write(' %d' % (Z))
+        fh.write('\n')
+        fh.write('#Enumerate different atomic species\n')
+        fh.write('typat')
+        fh.write('\n')
+        self.types = []
+        for Z in atoms.numbers:
+            for n, Zs in enumerate(self.species):
+                if Z == Zs:
+                    self.types.append(n+1)
+        n_entries_int = 20  # integer entries per line
+        for n, type in enumerate(self.types):
+            fh.write(' %d' % (type))
+            if n > 1 and ((n % n_entries_int) == 1):
+                fh.write('\n')
+        fh.write('\n')
+
+        fh.write('#Definition of the atoms\n')
+        fh.write('xangst\n')
+        for pos in atoms.positions:
+            fh.write('%.14f %.14f %.14f\n' %  tuple(pos))
+
+        if 'kptopt' not in param:
+            mp = kpts2mp(atoms, param.kpts)
+            fh.write('kptopt 1\n')
+            fh.write('ngkpt %d %d %d\n' % tuple(mp))
+            fh.write('nshiftk 1\n')
+            fh.write('shiftk\n')
+            fh.write('%.1f %.1f %.1f\n' % tuple((np.array(mp) + 1) % 2 * 0.5))
+
+        fh.write('chkexit 1 # abinit.exit file in the running directory terminates after the current SCF\n')
+
+        fh.close()
+
+    def read(self, label):
+        """Read results from ABINIT's text-output file."""
+        FileIOCalculator.read(self, label)
+        filename = self.label + '.txt'
+        if not os.path.isfile(filename):
+            raise ReadError
+
+        self.state = read_abinit(self.label + '.in')
+        self.parameters = Parameters.read(self.label + '.ase')
+
+        self.initialize(self.state)
+        self.read_results()
+
+    def read_results(self):
+        filename = self.label + '.txt'
+        text = open(filename).read().lower()
+        
+        if ('error' in text or
+            'was not enough scf cycles to converge' in text):
+            raise ReadError
+
+        for line in iter(text.split('\n')):
+            if line.rfind('natom  ') > -1:
+                natoms = int(line.split()[-1])
+
+        lines = iter(text.split('\n'))
+        # Stress:
+        # Printed in the output in the following format [Hartree/Bohr^3]:
+        # sigma(1 1)=  4.02063464E-04  sigma(3 2)=  0.00000000E+00
+        # sigma(2 2)=  4.02063464E-04  sigma(3 1)=  0.00000000E+00
+        # sigma(3 3)=  4.02063464E-04  sigma(2 1)=  0.00000000E+00
+        for line in lines:
+            if line.rfind(
+                'cartesian components of stress tensor (hartree/bohr^3)') > -1:
+                stress = np.empty(6)
+                for i in range(3):
+                    entries = lines.next().split()
+                    stress[i] = float(entries[2])
+                    stress[i + 3] = float(entries[5])
+                self.results['stress'] = stress * Hartree / Bohr**3
+                break
+        else:
+            raise RuntimeError
+
+        # Energy [Hartree]:
+        # Warning: Etotal could mean both electronic energy and free energy!
+        for line in iter(text.split('\n')):
+            if line.rfind('>>>>> internal e=') > -1:
+                etotal = float(line.split('=')[-1])*Hartree
+                for line1 in iter(text.split('\n')):
+                    if line1.rfind('>>>>>>>>> etotal=') > -1:
+                        efree = float(line1.split('=')[-1])*Hartree
+                        break
+                else:
+                    raise RuntimeError
+                break
+        else:
+            for line2 in iter(text.split('\n')):
+                if line2.rfind('>>>>>>>>> etotal=') > -1:
+                    etotal = float(line2.split('=')[-1])*Hartree
+                    efree = etotal
+                    break
+            else:
+                raise RuntimeError
+
+        # Energy extrapolated to zero Kelvin:
+        self.results['energy'] = (etotal + efree) / 2
+        self.results['free_energy'] = efree
+
+        # Forces:
+        for line in lines:
+            if line.rfind('cartesian forces (ev/angstrom) at end:') > -1:
+                forces = []
+                for i in range(natoms):
+                    forces.append(np.array(
+                            [float(f) for f in lines.next().split()[1:]]))
+                self.results['forces'] = np.array(forces)
+                break
+        else:
+            raise RuntimeError
+        #
+        self.nband = self.read_number_of_bands()
+        self.niter = self.read_number_of_iterations()
+        self.nelect = self.read_number_of_electrons()
+        self.results['magmom'] = self.read_magnetic_moment()
 
     def initialize(self, atoms):
-        self.numbers = atoms.get_atomic_numbers().copy()
+        numbers = atoms.get_atomic_numbers().copy()
         self.species = []
-        for a, Z in enumerate(self.numbers):
+        for a, Z in enumerate(numbers):
             if Z not in self.species:
                 self.species.append(Z)
 
-        if not hasattr(self, 'spinpol'):
-            self.spinpol = atoms.get_initial_magnetic_moments().any()
+        self.spinpol = atoms.get_initial_magnetic_moments().any()
 
         if 'ABINIT_PP_PATH' in os.environ:
             pppaths = os.environ['ABINIT_PP_PATH'].split(':')
@@ -123,29 +385,32 @@ class Abinit:
             pppaths = []
 
         self.ppp_list = []
-        if self.xc != 'LDA':
+        if self.parameters.xc != 'LDA':
             xcname = 'GGA'
         else:
             xcname = 'LDA'
 
+        pps = self.parameters.pps
+        if pps not in ['fhi', 'hgh', 'hgh.sc', 'hgh.k', 'tm', 'paw']:
+            raise ValueError('Unexpected PP identifier %s' % pps)
+
         for Z in self.species:
             symbol = chemical_symbols[abs(Z)]
             number = atomic_numbers[symbol]
 
-            pps = self.pps
             if pps == 'fhi':
                 name = '%02d-%s.%s.fhi' % (number, symbol, xcname)
             elif pps in ['paw']:
-                hghtemplate = '%s-%s-%s.paw' # E.g. "H-GGA-hard-uspp.paw"
+                hghtemplate = '%s-%s-%s.paw'  # E.g. "H-GGA-hard-uspp.paw"
                 name = hghtemplate % (symbol, xcname, '*')
             elif pps in ['hgh.k']:
-                hghtemplate = '%s-q%s.hgh.k' # E.g. "Co-q17.hgh.k"
+                hghtemplate = '%s-q%s.hgh.k'  # E.g. "Co-q17.hgh.k"
                 name = hghtemplate % (symbol, '*')
             elif pps in ['tm']:
-                hghtemplate = '%d%s%s.pspnc' # E.g. "44ru.pspnc"
+                hghtemplate = '%d%s%s.pspnc'  # E.g. "44ru.pspnc"
                 name = hghtemplate % (number, symbol.lower(), '*')
             elif pps in ['hgh', 'hgh.sc']:
-                hghtemplate = '%d%s.%s.hgh' # E.g. "42mo.6.hgh"
+                hghtemplate = '%d%s.%s.hgh'  # E.g. "42mo.6.hgh"
                 # There might be multiple files with different valence
                 # electron counts, so we must choose between
                 # the ordinary and the semicore versions for some elements.
@@ -164,29 +429,32 @@ class Abinit:
                         continue
                     assert len(filenames) in [0, 1, 2]
                     if pps == 'paw':
-                        selector = max # Semicore or hard
-                        # warning: see download.sh in 
-                        # abinit-pseudopotentials*tar.gz for additional information!
-                        S = selector([str(os.path.split(name)[1].split('-')[2][:-4])
-                                      for name in filenames])
+                        selector = max  # Semicore or hard
+                        # warning: see download.sh in
+                        # abinit-pseudopotentials*tar.gz for additional
+                        # information!
+                        S = selector(
+                            [str(os.path.split(name)[1].split('-')[2][:-4])
+                             for name in filenames])
                         name = hghtemplate % (symbol, xcname, S)
                     elif pps == 'hgh':
-                        selector = min # Lowest possible valence electron count
+                        selector = min  # Lowest valence electron count
                         Z = selector([int(os.path.split(name)[1].split('.')[1])
                                       for name in filenames])
                         name = hghtemplate % (number, symbol.lower(), str(Z))
                     elif pps == 'hgh.k':
-                        selector = min # Semicore - highest electron count
-                        Z = selector([int(os.path.split(name)[1].split('-')[1][:-6][1:])
-                                      for name in filenames])
+                        selector = min  # Semicore - highest electron count
+                        Z = selector(
+                            [int(os.path.split(name)[1].split('-')[1][:-6][1:])
+                             for name in filenames])
                         name = hghtemplate % (symbol, Z)
                     elif pps == 'tm':
-                        selector = max # Semicore - highest electron count
+                        selector = max  # Semicore - highest electron count
                         # currently only one version of psp per atom
                         name = hghtemplate % (number, symbol.lower(), '')
                     else:
                         assert pps == 'hgh.sc'
-                        selector = max # Semicore - highest electron count
+                        selector = max  # Semicore - highest electron count
                         Z = selector([int(os.path.split(name)[1].split('.')[1])
                                       for name in filenames])
                         name = hghtemplate % (number, symbol.lower(), str(Z))
@@ -198,17 +466,6 @@ class Abinit:
             if not found:
                 raise RuntimeError('No pseudopotential for %s!' % symbol)
 
-        self.converged = False
-
-    def get_potential_energy(self, atoms, force_consistent=False):
-        self.update(atoms)
-
-        if force_consistent:
-            return self.efree
-        else:
-            # Energy extrapolated to zero Kelvin:
-            return  (self.etotal + self.efree) / 2
-
     def get_number_of_iterations(self):
         return self.niter
 
@@ -220,7 +477,7 @@ class Abinit:
         return niter
 
     def get_electronic_temperature(self):
-        return self.width
+        return self.parameters.width
 
     def get_number_of_electrons(self):
         return self.nelect
@@ -228,8 +485,8 @@ class Abinit:
     def read_number_of_electrons(self):
         nelect = None
         # only in log file!
-        for line in open(self.label + '.log'): # find last one
-            if line.find('nelect') != -1:
+        for line in open(self.label + '.log'):  # find last one
+            if line.find('with nelect') != -1:
                 nelect = float(line.split('=')[1].strip())
         return nelect
 
@@ -261,10 +518,6 @@ class Abinit:
     def get_number_of_spins(self):
         return 1 + int(self.spinpol)
 
-    def get_magnetic_moment(self, atoms):
-        self.update(atoms)
-        return self.magnetic_moment
-
     def read_magnetic_moment(self):
         magmom = None
         if not self.get_spin_polarized():
@@ -275,14 +528,6 @@ class Abinit:
                     magmom = float(line.split('=')[-1].strip())
         return magmom
 
-    def get_magnetic_moments(self, atoms):
-        # local magnetic moments are not available in abinit
-        # so set the total magnetic moment on the atom no. 0 and fill with 0.0
-        self.update(atoms)
-        magmoms = [0.0 for a in range(len(atoms))]
-        magmoms[0] = self.get_magnetic_moment(atoms)
-        return np.array(magmoms)
-
     def get_fermi_level(self):
         return self.read_fermi()
 
@@ -292,202 +537,6 @@ class Abinit:
     def get_occupations(self, kpt=0, spin=0):
         return self.get_kpts_info(kpt, spin, 'occupations')
 
-    def get_forces(self, atoms):
-        self.update(atoms)
-        return self.forces.copy()
-
-    def get_stress(self, atoms):
-        self.update(atoms)
-        return self.stress.copy()
-
-    def calculate(self, atoms):
-        self.positions = atoms.get_positions().copy()
-        self.cell = atoms.get_cell().copy()
-        self.pbc = atoms.get_pbc().copy()
-
-        self.write_files()
-
-        self.write_inp(atoms)
-
-        # Now, because (stupidly) abinit when it finds a name it uses nameA
-        # and when nameA exists it uses nameB, etc.
-        # we need to rename our *.txt file to *.txt.bak
-        filename = self.label + '.txt'
-        if islink(filename) or isfile(filename):
-            os.rename(filename, filename+'.bak')
-
-        if 'ABINIT_SCRIPT' in os.environ:
-            abinit = os.environ['ABINIT_SCRIPT']
-            locals = {'label': self.label}
-            execfile(abinit, {}, locals)
-            exitcode = locals['exitcode']
-        else:
-            exitcode = os.system('abinis < %s.files > %s.log' %
-                                 (self.label, self.label))
-
-        if exitcode != 0:
-            raise RuntimeError(('Abinit exited with exit code: %d.  ' +
-                                'Check %s.log for more information.') %
-                               (exitcode, self.label))
-
-        self.read()
-
-        self.converged = True
-
-    def write_files(self):
-        """Write input parameters to files-file."""
-        fh = open(self.label + '.files', 'w')
-
-        import getpass
-        #find a suitable default scratchdir (should be writeable!)
-        username=getpass.getuser()
-
-        if os.access("/scratch/"+username,os.W_OK):
-                scratch = "/scratch/"+username
-        elif os.access("/scratch/",os.W_OK):
-                scratch = "/scratch/"
-        else:
-                if os.access(os.curdir,os.W_OK):
-                        scratch = os.curdir #if no /scratch use curdir
-                else:
-                        raise IOError,"No suitable scratch directory and no write access to current dir"
-
-        fh.write('%s\n' % (self.label+'.in')) # input
-        fh.write('%s\n' % (self.label+'.txt')) # output
-        fh.write('%s\n' % (self.label+'i')) # input
-        fh.write('%s\n' % (self.label+'o')) # output
-        # scratch files
-        scratchdir = os.path.dirname(os.path.join(scratch, self.label))
-        if not os.path.exists(scratchdir):
-            os.makedirs(scratchdir)
-        fh.write('%s\n' % (os.path.join(scratch, self.label+'.abinit')))
-        # Provide the psp files
-        for ppp in self.ppp_list:
-            fh.write('%s\n' % (ppp)) # psp file path
-
-        fh.close()
-
-    def set_inp(self, key, value):
-        """Set INPUT parameter."""
-        self.inp[key] = value
-
-    def write_inp(self, atoms):
-        """Write input parameters to in-file."""
-        fh = open(self.label + '.in', 'w')
-
-        inp = {
-            #'SystemLabel': self.label,
-            #'LatticeConstant': 1.0,
-            'natom': len(atoms),
-            'charge': self.charge,
-            #'DM.UseSaveDM': self.converged,
-            #'SolutionMethod': 'diagon',
-            'npulayit': self.pulay, # default 7
-            'diemix': self.mix
-            }
-
-        if not self.nband is None:
-            inp.update({'nband': self.nband})
-
-        if not self.nstep is None:
-            inp.update({'nstep': self.nstep})
-
-        if self.ecut is not None:
-            inp['ecut'] = str(self.ecut)+' eV' # default Ha
-
-        if self.width is not None:
-            inp['tsmear'] = str(self.width)+' eV' # default Ha
-            fh.write('occopt 3 # Fermi-Dirac smearing\n')
-
-        inp['ixc'] = { # default 1
-            'LDA':     7,
-            'PBE':    11,
-            'revPBE': 14,
-            'RPBE':   15,
-            'WC':     23
-            }[self.xc]
-
-        magmoms = atoms.get_initial_magnetic_moments()
-        if magmoms.any():
-            inp['nsppol'] = 2
-            fh.write('spinat\n')
-            for n, M in enumerate(magmoms):
-                fh.write('%.14f %.14f %.14f\n' % (0, 0, M))
-        else:
-            inp['nsppol'] = 1
-
-        inp.update(self.inp)
-
-        for key, value in inp.items():
-            if value is None:
-                continue
-
-            if isinstance(value, list):
-                fh.write('%block %s\n' % key)
-                for line in value:
-                    fh.write(' '.join(['%s' % x for x in line]) + '\n')
-                fh.write('%endblock %s\n' % key)
-
-            unit = keys_with_units.get(inpify(key))
-            if unit is None:
-                fh.write('%s %s\n' % (key, value))
-            else:
-                if 'fs**2' in unit:
-                    value /= fs**2
-                elif 'fs' in unit:
-                    value /= fs
-                fh.write('%s %f %s\n' % (key, value, unit))
-
-        fh.write('#Definition of the unit cell\n')
-        fh.write('acell\n')
-        fh.write('%.14f %.14f %.14f Angstrom\n' %  (1.0, 1.0, 1.0))
-        fh.write('rprim\n')
-        for v in self.cell:
-            fh.write('%.14f %.14f %.14f\n' %  tuple(v))
-
-        fh.write('chkprim 0 # Allow non-primitive cells\n')
-
-        fh.write('#Definition of the atom types\n')
-        fh.write('ntypat %d\n' % (len(self.species)))
-        fh.write('znucl')
-        for n, Z in enumerate(self.species):
-            fh.write(' %d' % (Z))
-        fh.write('\n')
-        fh.write('#Enumerate different atomic species\n')
-        fh.write('typat')
-        fh.write('\n')
-        self.types = []
-        for Z in self.numbers:
-            for n, Zs in enumerate(self.species):
-                if Z == Zs:
-                    self.types.append(n+1)
-        for n, type in enumerate(self.types):
-            fh.write(' %d' % (type))
-            if n > 1 and ((n % self.n_entries_int) == 1):
-                fh.write('\n')
-        fh.write('\n')
-
-        fh.write('#Definition of the atoms\n')
-        fh.write('xangst\n')
-        a = 0
-        for pos, Z in zip(self.positions, self.numbers):
-            a += 1
-            fh.write('%.14f %.14f %.14f\n' %  tuple(pos))
-
-        if self.kpts is not None:
-            fh.write('kptopt 1\n')
-            fh.write('ngkpt %d %d %d\n' % tuple(self.kpts))
-            fh.write('nshiftk 1\n')
-            fh.write('shiftk\n')
-            fh.write('%.1f %.1f %.1f\n' %
-                     tuple((np.array(self.kpts) + 1) % 2 * 0.5))
-
-        fh.write('#Definition of the SCF procedure\n')
-        fh.write('toldfe %.1g\n' %  self.toldfe)
-        fh.write('chkexit 1 # abinit.exit file in the running directory terminates after the current SCF\n')
-
-        fh.close()
-
     def read_fermi(self):
         """Method that reads Fermi energy in Hartree from the output file
         and returns it in eV"""
@@ -523,8 +572,9 @@ class Abinit:
            spinname = 'SPIN UP'.lower()
         # number of lines of eigenvalues/occupations for a kpt
         nband = self.get_number_of_bands()
-        n_entry_lines = max(1, int((nband - 0.1)/self.n_entries_float) + 1)
-        #
+        n_entries_float = 8  # float entries per line
+        n_entry_lines = max(1, int((nband - 0.1) / n_entries_float) + 1)
+
         filename = self.label + '.txt'
         text = open(filename).read().lower()
         assert 'error' not in text
@@ -573,7 +623,7 @@ class Abinit:
             range_kpts = 2*n_kpts
         else:
             range_kpts = n_kpts
-        #
+
         values_list = []
         offset = 0
         for kpt_entry in range(range_kpts):
@@ -598,134 +648,8 @@ class Abinit:
                         full_line = float(full_line.split('wtk=')[1].strip().split(',')[0].split()[0])
                     values_list.append(full_line)
             offset = offset+n_entry_lines+1
-        #
+
         if mode in ['occupations', 'eigenvalues']:
             return np.array(values_list[kpt])
         else:
             return np.array(values_list)
-
-    def read(self):
-        """Read results from ABINIT's text-output file."""
-        filename = self.label + '.txt'
-        text = open(filename).read().lower()
-        assert 'error' not in text
-        assert 'was not enough scf cycles to converge' not in text
-        # some consistency ckecks
-        for line in iter(text.split('\n')):
-            if line.rfind('natom  ') > -1:
-                natom = int(line.split()[-1])
-                assert natom == len(self.numbers)
-        for line in iter(text.split('\n')):
-            if line.rfind('znucl  ') > -1:
-                znucl = [float(Z) for Z in line.split()[1:]]
-                for n, Z in enumerate(self.species):
-                    assert Z == znucl[n]
-        lines = text.split('\n')
-        for n, line in enumerate(lines):
-            if line.rfind(' typat  ') > -1:
-                nlines = len(self.numbers) / self.n_entries_int
-                typat = [int(t) for t in line.split()[1:]] # first line
-                for nline in range(nlines): # remaining lines
-                    for t in lines[1 + n + nline].split()[:]:
-                        typat.append(int(t))
-        for n, t in enumerate(self.types):
-            assert t == typat[n]
-
-        lines = iter(text.split('\n'))
-        # Stress:
-        # Printed in the output in the following format [Hartree/Bohr^3]:
-        # sigma(1 1)=  4.02063464E-04  sigma(3 2)=  0.00000000E+00
-        # sigma(2 2)=  4.02063464E-04  sigma(3 1)=  0.00000000E+00
-        # sigma(3 3)=  4.02063464E-04  sigma(2 1)=  0.00000000E+00
-        for line in lines:
-            if line.rfind('cartesian components of stress tensor (hartree/bohr^3)') > -1:
-                self.stress = np.empty((3, 3))
-                for i in range(3):
-                    entries = lines.next().split()
-                    self.stress[i,i] = float(entries[2])
-                    self.stress[min(3, 4-i)-1, max(1, 2-i)-1] = float(entries[5])
-                    self.stress[max(1, 2-i)-1, min(3, 4-i)-1] = float(entries[5])
-                self.stress = self.stress*Hartree/Bohr**3
-                break
-        else:
-            raise RuntimeError
-
-        # Energy [Hartree]:
-        # Warning: Etotal could mean both electronic energy and free energy!
-        for line in iter(text.split('\n')):
-            if line.rfind('>>>>> internal e=') > -1:
-                self.etotal = float(line.split('=')[-1])*Hartree
-                for line1 in iter(text.split('\n')):
-                    if line1.rfind('>>>>>>>>> etotal=') > -1:
-                        self.efree = float(line1.split('=')[-1])*Hartree
-                        break
-                else:
-                    raise RuntimeError
-                break
-        else:
-            for line2 in iter(text.split('\n')):
-                if line2.rfind('>>>>>>>>> etotal=') > -1:
-                    self.etotal = float(line2.split('=')[-1])*Hartree
-                    self.efree = self.etotal
-                    break
-            else:
-                raise RuntimeError
-
-        # Forces:
-        for line in lines:
-            if line.rfind('cartesian forces (ev/angstrom) at end:') > -1:
-                forces = []
-                for i in range(len(self.numbers)):
-                    forces.append(np.array([float(f) for f in lines.next().split()[1:]]))
-                self.forces = np.array(forces)
-                break
-        else:
-            raise RuntimeError
-        #
-        self.nband = self.read_number_of_bands()
-        self.niter = self.read_number_of_iterations()
-        self.nelect = self.read_number_of_electrons()
-        self.magnetic_moment = self.read_magnetic_moment()
-
-def inpify(key):
-    return key.lower().replace('_', '').replace('.', '').replace('-', '')
-
-
-keys_with_units = {
-    }
-#keys_with_units = {
-#    'paoenergyshift': 'eV',
-#    'zmunitslength': 'Bohr',
-#    'zmunitsangle': 'rad',
-#    'zmforcetollength': 'eV/Ang',
-#    'zmforcetolangle': 'eV/rad',
-#    'zmmaxdispllength': 'Ang',
-#    'zmmaxdisplangle': 'rad',
-#    'ecut': 'eV',
-#    'dmenergytolerance': 'eV',
-#    'electronictemperature': 'eV',
-#    'oneta': 'eV',
-#    'onetaalpha': 'eV',
-#    'onetabeta': 'eV',
-#    'onrclwf': 'Ang',
-#    'onchemicalpotentialrc': 'Ang',
-#    'onchemicalpotentialtemperature': 'eV',
-#    'mdmaxcgdispl': 'Ang',
-#    'mdmaxforcetol': 'eV/Ang',
-#    'mdmaxstresstol': 'eV/Ang**3',
-#    'mdlengthtimestep': 'fs',
-#    'mdinitialtemperature': 'eV',
-#    'mdtargettemperature': 'eV',
-#    'mdtargetpressure': 'eV/Ang**3',
-#    'mdnosemass': 'eV*fs**2',
-#    'mdparrinellorahmanmass': 'eV*fs**2',
-#    'mdtaurelax': 'fs',
-#    'mdbulkmodulus': 'eV/Ang**3',
-#    'mdfcdispl': 'Ang',
-#    'warningminimumatomicdistance': 'Ang',
-#    'rcspatial': 'Ang',
-#    'kgridcutoff': 'Ang',
-#    'latticeconstant': 'Ang'}
-
-# shortcut function names
-Abinit.get_occupation_numbers = Abinit.get_occupations
