@@ -1,10 +1,16 @@
 import os
+import warnings
+# cmr calls all available methods in ase.atoms detected by the module inspect.
+# Therefore also deprecated methods are called - and we choose to silence those warnings.
+warnings.filterwarnings('ignore', 'ase.atoms.*deprecated',)
 
 from ase.test import NotAvailable
 
+# if CMR_SETTINGS_FILE is missing, cmr raises simply
+# Exception("CMR is not configured properly. Please create the settings file with cmr --create-settings.")
 try:
     import cmr
-except ImportError:
+except (Exception, ImportError):
     raise NotAvailable('CMR is required')
 
 from ase.calculators.emt import EMT
