@@ -1,4 +1,8 @@
 import os
+import warnings
+# cmr calls all available methods in ase.atoms detected by the module inspect.
+# Therefore also deprecated methods are called - and we choose to silence those warnings.
+warnings.filterwarnings('ignore', 'ase.atoms.*deprecated',)
 
 import numpy as np
 
@@ -7,10 +11,12 @@ def array_almost_equal(a1, a2, tol=np.finfo(type(1.0)).eps):
     return (np.abs(a1 - a2) < tol).all()
 
 from ase.test import NotAvailable
-# this test should be run with cmr!
+
+# if CMR_SETTINGS_FILE is missing, cmr raises simply
+# Exception("CMR is not configured properly. Please create the settings file with cmr --create-settings.")
 try:
     import cmr
-except ImportError:
+except (Exception, ImportError):
     raise NotAvailable('CMR is required')
 
 from ase.calculators.emt import EMT
