@@ -18,6 +18,3 @@ b1 = nc.variables['matrix1'][:]
 b2 = nc.variables['matrix2'][:]
 
 assert np.all(a1 == b1) and np.all(a2 == b2)
-
-import os
-os.remove('test.nc')
