@@ -406,29 +406,56 @@ def sort(atoms, tags=None):
     return atoms[indices]
 
 
-def rotate(atoms, a1, a2, b1, b2, rotate_cell=True):
-    """Rotate *atoms*, such that *a1* will be rotated to *a2* and *b1*
-    to *b2*."""
+def rotation_matrix(a1, a2, b1, b2):
+    """Returns a rotation matrix that rotates the vectors *a1* in the
+    direction of *a2* and *b1* in the direction of *b2*.
+    
+    In the case that the angle between *a2* and *b2* is not the same
+    as between *a1* and *b1*, a proper rotation matrix will anyway be
+    constructed by first rotate *b2* in the *b1*, *b2* plane.
+    """
     from numpy.linalg import norm, det
     a1 = np.asarray(a1, dtype=float)/norm(a1)
-    a2 = np.asarray(a2, dtype=float)/norm(a2)
     b1 = np.asarray(b1, dtype=float)/norm(b1)
+    c1 = np.cross(a1, b1)
+    c1 /= norm(c1)      # clean out rounding errors...
+
+    a2 = np.asarray(a2, dtype=float)/norm(a2)
     b2 = np.asarray(b2, dtype=float)/norm(b2)
-    if norm(a2 - a1) < 1e-5:
-        n = 0.5*(a1 + a2)
-        a1, a2 = b1, b2
-    elif norm(b2 - b1) < 1e-5:
-        n = 0.5*(b1 + b2)
-    else:
-        n = np.cross(a2 - a1, b2 - b1)
-    n /= norm(n)
-    ap1 = a1 - np.dot(a1, n)*n
-    ap2 = a2 - np.dot(a2, n)*n
-    angle = np.arccos(np.dot(ap1, ap2)/(norm(ap1)*norm(ap2)))
-    angle *= np.sign(det((ap1, ap2, n)))
-    atoms.rotate(n, angle, rotate_cell=rotate_cell)
+    c2 = np.cross(a2, b2)
+    c2 /= norm(c2)      # clean out rounding errors...
+
+    # Calculate rotated *b2*
+    theta = np.arccos(np.dot(a2, b2)) - np.arccos(np.dot(a1, b1))
+    b3 = np.sin(theta)*a2 + np.cos(theta)*b2
+    b3 /= norm(b3)      # clean out rounding errors...
+    
+    A1 = np.array([a1, b1, c1])
+    A2 = np.array([a2, b3, c2])
+    R = np.linalg.solve(A1, A2).T
+    return R
 
 
+def rotate(atoms, a1, a2, b1, b2, rotate_cell=True, center=(0, 0, 0)):
+    """Rotate *atoms*, such that *a1* will be rotated in the direction
+    of *a2* and *b1* in the direction of *b2*.  The point at *center*
+    is fixed.  Use *center='COM'* to fix the center of mass.  If
+    *rotate_cell* is true, the cell will be rotated together with the
+    atoms.
+
+    Note that the 000-corner of the cell is by definition fixed at
+    origo.  Hence, setting *center* to something other than (0, 0, 0)
+    will rotate the atoms out of the cell, even if *rotate_cell* is
+    True.
+    """
+    if isinstance(center, str) and center.lower() == 'com':
+        center = atoms.get_center_of_mass()
+    
+    R = rotation_matrix(a1, a2, b1, b2)
+    atoms.positions[:] = np.dot(atoms.positions - center, R.T) + center
+
+    if rotate_cell:
+        atoms.cell[:] = np.dot(atoms.cell, R.T)
 
 #-----------------------------------------------------------------
 # Self test
