@@ -20,7 +20,7 @@ for i, a in enumerate([cnt1, cnt2]):
           show_unit_cell=2, display=False, run_povray=True)
 
 ind = [2, 0, 1]
-gnr1 = graphene_nanoribbon(3, 4, type='armchair')
+gnr1 = graphene_nanoribbon(3, 4, type='armchair', saturated=True)
 gnr1.set_cell(np.diag(gnr1.cell)[ind])
 gnr1.positions = gnr1.positions[:, ind]
 gnr2 = graphene_nanoribbon(2, 6, type='zigzag', saturated=True,
